@@ -1,7 +1,7 @@
 
 #   IGraph R package
-#   Copyright (C) 2003-2008  Gabor Csardi <csardi@rmki.kfki.hu>
-#   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+#   Copyright (C) 2003-2012  Gabor Csardi <csardi.gabor@gmail.com>
+#   334 Harvard street, Cambridge, MA 02139 USA
 #   
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -40,7 +40,7 @@
       vsize.from <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       res <- cbind(coords[,1] + vsize.from*cos(phi),
                    coords[,2] + vsize.from*sin(phi) )
@@ -50,7 +50,7 @@
       vsize.to <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       res <- cbind(coords[,1] + (r-vsize.to)*cos(phi),
                    coords[,2] + (r-vsize.to)*sin(phi) )
@@ -60,12 +60,12 @@
       vsize.from <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       vsize.to <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       res <- cbind(coords[,1] + vsize.from*cos(phi),
                    coords[,2] + vsize.from*sin(phi),
@@ -81,15 +81,15 @@
   } else if (mode=="plot") {
     vertex.color       <- params("vertex", "color")
     if (length(vertex.color) != 1 && !is.null(v)) {
-      vertex.color <- vertex.color[v+1]
+      vertex.color <- vertex.color[v]
     }
     vertex.frame.color <- params("vertex", "frame.color")
     if (length(vertex.frame.color) != 1 && !is.null(v)) {
-      vertex.frame.color <- vertex.frame.color[v+1]
+      vertex.frame.color <- vertex.frame.color[v]
     }
     vertex.size        <- 1/200 * params("vertex", "size")
     if (length(vertex.size) != 1 && !is.null(v)) {
-      vertex.size <- vertex.size[v+1]
+      vertex.size <- vertex.size[v]
     }
     vertex.size <- rep(vertex.size, length=nrow(coords))
     
@@ -114,9 +114,9 @@
 
     square.shift <- function(x0, y0, x1, y1, vsize) {
       m <- (y0-y1)/(x0-x1)
-      l <- cbind((-vsize+m*x1)/m , y1-vsize,
+      l <- cbind(x1-vsize/m , y1-vsize,
                  x1-vsize , y1-vsize*m,
-                 (vsize+m*x1)/m , y1+vsize,
+                 x1+vsize/m, y1+vsize,
                  x1+vsize , y1+vsize*m )
       
       v <- cbind(x1-vsize <= l[,1] & l[,1] <= x1+vsize &
@@ -144,7 +144,7 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       res <- res1 <- square.shift(coords[,3], coords[,4], coords[,1], coords[,2],
                                   vsize)
@@ -153,7 +153,7 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       res <- res2 <- square.shift(coords[,1], coords[,2], coords[,3], coords[,4],
                                   vsize)
@@ -170,15 +170,15 @@
   } else if (mode=="plot") {
     vertex.color       <- params("vertex", "color")
     if (length(vertex.color) != 1 && !is.null(v)) {
-      vertex.color <- vertex.color[v+1]
+      vertex.color <- vertex.color[v]
     }
     vertex.frame.color <- params("vertex", "frame.color")
     if (length(vertex.frame.color) != 1 && !is.null(v)) {
-      vertex.frame.color <- vertex.frame.color[v+1]
+      vertex.frame.color <- vertex.frame.color[v]
     }
     vertex.size        <- 1/200 * params("vertex", "size")
     if (length(vertex.size) != 1 && !is.null(v)) {
-      vertex.size <- vertex.size[v+1]
+      vertex.size <- vertex.size[v]
     }
     vertex.size <- rep(vertex.size, length=nrow(coords))
 
@@ -224,7 +224,7 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       res <- res1 <- square.shift(coords[,3], coords[,4], coords[,1], coords[,2],
                                   vsize)
@@ -233,7 +233,7 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       res <- res2 <- square.shift(coords[,1], coords[,2], coords[,3], coords[,4],
                                   vsize)
@@ -302,12 +302,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res1 <- rec.shift(coords[,3], coords[,4], coords[,1], coords[,2],
                                vsize, vsize2)
@@ -316,12 +316,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res2 <- rec.shift(coords[,1], coords[,2], coords[,3], coords[,4],
                                vsize, vsize2)
@@ -339,20 +339,20 @@
 
     vertex.color       <- params("vertex", "color")
     if (length(vertex.color) != 1 && !is.null(v)) {
-      vertex.color <- vertex.color[v+1]
+      vertex.color <- vertex.color[v]
     }
     vertex.frame.color <- params("vertex", "frame.color")
     if (length(vertex.frame.color) != 1 && !is.null(v)) {
-      vertex.frame.color <- vertex.frame.color[v+1]
+      vertex.frame.color <- vertex.frame.color[v]
     }
     vertex.size        <- 1/200 * params("vertex", "size")
     if (length(vertex.size) != 1 && !is.null(v)) {
-      vertex.size <- vertex.size[v+1]
+      vertex.size <- vertex.size[v]
     }
     vertex.size <- rep(vertex.size, length=nrow(coords))   
     vertex.size2       <- 1/200 * params("vertex", "size2")
     if (length(vertex.size2) != 1 && !is.null(v)) {
-      vertex.size2 <- vertex.size2[v+1]
+      vertex.size2 <- vertex.size2[v]
     }
     vertex.size <- cbind(vertex.size, vertex.size2)
 
@@ -399,12 +399,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res1 <- rec.shift(coords[,3], coords[,4], coords[,1], coords[,2],
                                vsize, vsize2)
@@ -413,12 +413,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res2 <- rec.shift(coords[,1], coords[,2], coords[,3], coords[,4],
                                vsize, vsize2)
@@ -471,12 +471,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,1]+1 ]
+        vertex.size[ el[,1] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res1 <- rec.shift(coords[,3], coords[,4], coords[,1], coords[,2],
                                vsize, vsize2)
@@ -485,12 +485,12 @@
       vsize <- if (length(vertex.size)==1) {
         vertex.size
       } else {
-        vertex.size[ el[,2]+1 ]
+        vertex.size[ el[,2] ]
       }
       vsize2 <- if (length(vertex.size2)==1) {
         vertex.size2
       } else {
-        vertex.size2[ el[,1]+1 ]
+        vertex.size2[ el[,1] ]
       }
       res <- res2 <- rec.shift(coords[,1], coords[,2], coords[,3], coords[,4],
                                vsize, vsize2)
@@ -526,11 +526,135 @@
     invisible(NULL)
   }
 }
+
+mypie <- function(x, y, values, radius, edges=200, col=NULL, angle=45,
+                  density=NULL, border=NULL, lty=NULL, init.angle=90, ...) {
+  values <- c(0, cumsum(values)/sum(values))
+  dx <- diff(values)
+  nx <- length(dx)
+  twopi <- 2 * pi
+  if (is.null(col)) 
+    col <- if (is.null(density)) 
+      c("white", "lightblue", "mistyrose", "lightcyan", 
+        "lavender", "cornsilk")
+    else par("fg")
+  col <- rep(col, length.out = nx)
+  border <- rep(border, length.out = nx)
+  lty <- rep(lty, length.out = nx)
+  angle <- rep(angle, length.out = nx)
+  density <- rep(density, length.out = nx)
+  t2xy <- function(t) {
+    t2p <- twopi * t + init.angle * pi/180
+    list(x = radius * cos(t2p), y = radius * sin(t2p))
+  }
+  for (i in 1:nx) {
+    n <- max(2, floor(edges * dx[i]))
+    P <- t2xy(seq.int(values[i], values[i + 1], length.out = n))
+    polygon(x+c(P$x, 0), y+c(P$y, 0), density = density[i], angle = angle[i], 
+            border = border[i], col = col[i], lty = lty[i], ...)
+  }
+}
+
+.igraph.shape.pie <- function(coords, el=NULL, v=NULL, mode=c("clip", "plot"),
+                                 params, end=c("both", "from", "to")) {
+
+  mode=match.arg(mode)
+  end =match.arg(end)
+
+  #####################################################################
+  ## clipping mode, this is the same as for circle
+  
+  if (mode=="clip") {
+    if (length(coords)==0) { return (coords) }     
+
+    vertex.size <- 1/200 * params("vertex", "size")
+
+    if (end=="from") {
+      phi <- atan2(coords[,4] - coords[,2], coords[,3] - coords[,1])
+      vsize.from <- if (length(vertex.size)==1) {
+        vertex.size
+      } else {
+        vertex.size[ el[,1] ]
+      }
+      res <- cbind(coords[,1] + vsize.from*cos(phi),
+                   coords[,2] + vsize.from*sin(phi) )
+    } else if (end=="to") {
+      phi <- atan2(coords[,4] - coords[,2], coords[,3] - coords[,1])
+      r <- sqrt( (coords[,3] - coords[,1])^2 + (coords[,4] - coords[,2])^2 )
+      vsize.to <- if (length(vertex.size)==1) {
+        vertex.size
+      } else {
+        vertex.size[ el[,2] ]
+      }
+      res <- cbind(coords[,1] + (r-vsize.to)*cos(phi),
+                   coords[,2] + (r-vsize.to)*sin(phi) )
+    } else if (end=="both") {
+      phi <- atan2(coords[,4] - coords[,2], coords[,3] - coords[,1])
+      r <- sqrt( (coords[,3] - coords[,1])^2 + (coords[,4] - coords[,2])^2 )
+      vsize.from <- if (length(vertex.size)==1) {
+        vertex.size
+      } else {
+        vertex.size[ el[,1] ]
+      }
+      vsize.to <- if (length(vertex.size)==1) {
+        vertex.size
+      } else {
+        vertex.size[ el[,2] ]
+      }
+      res <- cbind(coords[,1] + vsize.from*cos(phi),
+                   coords[,2] + vsize.from*sin(phi),
+                   coords[,1] + (r-vsize.to)*cos(phi),
+                   coords[,2] + (r-vsize.to)*sin(phi) )
+    }
+
+    res
+
+  #####################################################################
+  ## plotting mode
     
+  } else if (mode=="plot") {
+    getparam <- function(pname) {
+      p <- params("vertex", pname)
+      if (length(p) != 1 && !is.null(v)) {
+        p <- p[v]
+      }
+      p
+    }
+    vertex.color       <- getparam("color")
+    vertex.frame.color <- getparam("frame.color")
+    vertex.size        <- rep(1/200 * getparam("size"), length=nrow(coords))
+    vertex.pie         <- getparam("pie")
+    vertex.pie.color   <- getparam("pie.color")
+    vertex.pie.angle   <- getparam("pie.angle")
+    vertex.pie.density <- getparam("pie.density")
+    vertex.pie.lty     <- getparam("pie.lty")
+
+    for (i in seq_len(nrow(coords))) {
+      pie <- if(length(vertex.pie)==1) {
+        vertex.pie
+      } else {
+        vertex.pie[[i]]
+      }
+      col <- if (length(vertex.pie.color)==1) {
+        vertex.pie.color[[1]]
+      } else {
+        vertex.pie.color[[i]]
+      }
+      mypie(x=coords[i,1], y=coords[i,2], pie,
+            radius=vertex.size[i], edges=200, col=col,
+            angle=na.omit(vertex.pie.angle[c(i,1)])[1],
+            density=na.omit(vertex.pie.density[c(i,1)])[1],
+            border=na.omit(vertex.frame.color[c(i,1)])[1],
+            lty=na.omit(vertex.pie.lty[c(i,1)])[1])
+    }
+  }
+}
+
 .igraph.shapes <- list("circle" = .igraph.shape.circle,
                        "square" = .igraph.shape.square,
                        "csquare" = .igraph.shape.csquare,
                        "rectangle" = .igraph.shape.rectangle,
                        "crectangle" = .igraph.shape.crectangle,
                        "vrectangle" = .igraph.shape.vrectangle,
-                       "none"   = .igraph.shape.none)
+                       "none"   = .igraph.shape.none,
+                       "pie"    = .igraph.shape.pie)
