@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2003, 2004, 2005  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2003-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,10 +21,11 @@
 
 */
 
-#include "types.h"
-#include "memory.h"
-#include "random.h"
-#include "error.h"
+#include "igraph_types.h"
+#include "igraph_strvector.h"
+#include "igraph_memory.h"
+#include "igraph_random.h"
+#include "igraph_error.h"
 #include "config.h"
 
 #include <assert.h>
@@ -37,8 +38,12 @@
  * The current implementation is very simple and not too efficient. It
  * works fine for not too many strings, e.g. the list of attribute
  * names is returned in a string vector by \ref
- * igraph_cattribute_list(). Do not expect great performace from this
+ * igraph_cattribute_list(). Do not expect great performance from this
  * type.</para>
+ * 
+ * <para>
+ * \example examples/simple/igraph_strvector.c
+ * </para>
  */
 
 /**
@@ -268,7 +273,7 @@ void igraph_strvector_move_interval(igraph_strvector_t *v, long int begin,
   }
   for (i=0; i<end-begin; i++) {
     if (v->data[begin+i] != 0) {
-      long int len=strlen(v->data[begin+i])+1;
+      size_t len=strlen(v->data[begin+i])+1;
       v->data[to+i]=igraph_Calloc(len, char);
       memcpy(v->data[to+i], v->data[begin+i], sizeof(char)*len);
     }
@@ -318,7 +323,7 @@ int igraph_strvector_copy(igraph_strvector_t *to,
  * Concatenate two string vectors.
  * 
  * \param to The first string vector, the result is stored here.
- * \param from The second string vector, it is kep unchanged.
+ * \param from The second string vector, it is kept unchanged.
  * \return Error code.
  * 
  * Time complexity: O(n+l2), n is the number of strings in the new
@@ -394,17 +399,17 @@ void igraph_strvector_clear(igraph_strvector_t *sv) {
 int igraph_strvector_resize(igraph_strvector_t* v, long int newsize) {
   long int toadd=newsize-v->len, i, j;
   char **tmp;
+  long int reallocsize=newsize;
+  if (reallocsize==0) { reallocsize=1; }
   
   assert(v != 0);
   assert(v->data != 0);
 /*   printf("resize %li to %li\n", v->len, newsize); */
   if (newsize < v->len) { 
-    long int i, reallocsize=newsize;
     for (i=newsize; i<v->len; i++) {
       igraph_Free(v->data[i]);
     }
     /* try to give back some space */
-    if (reallocsize==0) { reallocsize=1; }
     tmp=igraph_Realloc(v->data, reallocsize, char*);
 /*     printf("resize %li to %li, %p\n", v->len, newsize, tmp); */
     if (tmp != 0) {
@@ -412,7 +417,7 @@ int igraph_strvector_resize(igraph_strvector_t* v, long int newsize) {
     }
   } else if (newsize > v->len) {
     igraph_bool_t error=0;
-    tmp=igraph_Realloc(v->data, newsize, char*);
+    tmp=igraph_Realloc(v->data, reallocsize, char*);
     if (tmp==0) {
       IGRAPH_ERROR("cannot resize string vector", IGRAPH_ENOMEM);
     }
@@ -438,7 +443,7 @@ int igraph_strvector_resize(igraph_strvector_t* v, long int newsize) {
       if (tmp != 0) {
 	v->data=tmp;
       }
-      IGRAPH_ERROR("canot resize string vector", IGRAPH_ENOMEM);
+      IGRAPH_ERROR("Cannot resize string vector", IGRAPH_ENOMEM);
     }
   }
   v->len = newsize;
@@ -516,8 +521,8 @@ void igraph_strvector_permdelete(igraph_strvector_t *v, const igraph_vector_t *i
       igraph_Free(v->data[i]);
     }
   }
-  /* Try to make it shorter */
-  tmp=igraph_Realloc(v->data, v->len-nremove, char*);
+  /* Try to make it shorter */  
+  tmp=igraph_Realloc(v->data, v->len-nremove ? v->len-nremove : 1, char*);
   if (tmp != 0) {
     v->data=tmp;
   }
@@ -544,10 +549,40 @@ void igraph_strvector_remove_negidx(igraph_strvector_t *v, const igraph_vector_t
     }
   }
   /* Try to give back some memory */
-  tmp=igraph_Realloc(v->data, v->len-nremove, char*);
+  tmp=igraph_Realloc(v->data, v->len-nremove ? v->len-nremove : 1, char*);
   if (tmp != 0) {
     v->data=tmp;
   }
   v->len -= nremove;
 }
 
+int igraph_strvector_print(const igraph_strvector_t *v, FILE *file,
+			   const char *sep) {
+
+  long int i, n=igraph_strvector_size(v);
+  if (n!=0) {
+    fprintf(file, "%s", STR(*v, 0));
+  }
+  for (i=1; i<n; i++) {
+    fprintf(file, "%s%s", sep, STR(*v, i));
+  }
+  return 0;
+
+}
+
+int igraph_strvector_index(const igraph_strvector_t *v, 
+                           igraph_strvector_t *newv,
+                           const igraph_vector_t *idx) {
+  
+  long int i, newlen=igraph_vector_size(idx);
+  IGRAPH_CHECK(igraph_strvector_resize(newv, newlen));
+
+  for (i=0; i<newlen; i++) {
+    long int j=VECTOR(*idx)[i];
+    char *str;
+    igraph_strvector_get(v, j, &str);
+    igraph_strvector_set(newv, i, str);
+  }
+  
+  return 0;
+}
