@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,13 +21,43 @@
 
 */
 
-#include "igraph.h"
-#include "memory.h"
-#include "random.h"
+#include "igraph_motifs.h"
+#include "igraph_memory.h"
+#include "igraph_random.h"
+#include "igraph_adjlist.h"
+#include "igraph_interrupt_internal.h"
+#include "igraph_interface.h"
+#include "igraph_nongraph.h"
+#include "igraph_structural.h"
+#include "igraph_stack.h"
 #include "config.h"
 
 #include <string.h>
 
+extern unsigned int igraph_i_isoclass_3[];
+extern unsigned int igraph_i_isoclass_4[];
+extern unsigned int igraph_i_isoclass_3u[];
+extern unsigned int igraph_i_isoclass_4u[];
+extern unsigned int igraph_i_isoclass2_3[];
+extern unsigned int igraph_i_isoclass2_4[];
+extern unsigned int igraph_i_isoclass2_3u[];
+extern unsigned int igraph_i_isoclass2_4u[];
+extern unsigned int igraph_i_isoclass_3_idx[];
+extern unsigned int igraph_i_isoclass_4_idx[];
+extern unsigned int igraph_i_isoclass_3u_idx[];
+extern unsigned int igraph_i_isoclass_4u_idx[];
+
+/**
+ * Callback function for igraph_motifs_randesu that counts the motifs by
+ * isomorphism class in a histogram.
+ */
+igraph_bool_t igraph_i_motifs_randesu_update_hist(const igraph_t *graph,
+    igraph_vector_t *vids, int isoclass, void* extra) {
+  igraph_vector_t *hist = (igraph_vector_t*)extra;
+  VECTOR(*hist)[isoclass]++;
+  return 0;
+}
+
 /**
  * \function igraph_motifs_randesu
  * \brief Count the number of motifs in a graph
@@ -76,13 +106,74 @@
  * \sa \ref igraph_motifs_randesu_estimate() for estimating the number
  * of motifs in a graph, this can help to set the \c cut_prob
  * parameter; \ref igraph_motifs_randesu_no() to calculate the total
- * number of motifs of a given size in a graph.
+ * number of motifs of a given size in a graph;
+ * \ref igraph_motifs_randesu_callback() for calling a callback function
+ * for every motif found.
  * 
  * Time complexity: TODO.
+ * 
+ * \example examples/simple/igraph_motifs_randesu.c
  */
-
 int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist, 
 			  int size, const igraph_vector_t *cut_prob) {
+  int histlen;
+
+  if (size != 3 && size != 4) {
+    IGRAPH_ERROR("Only 3 and 4 vertex motifs are implemented",
+		 IGRAPH_EINVAL);
+  }
+  if (size==3) {
+    histlen = igraph_is_directed(graph) ? 16 : 4;
+  } else {
+    histlen = igraph_is_directed(graph) ? 218 : 11;
+  }
+
+  IGRAPH_CHECK(igraph_vector_resize(hist, histlen));
+  igraph_vector_null(hist);
+
+  IGRAPH_CHECK(igraph_motifs_randesu_callback(graph, size, cut_prob,
+	&igraph_i_motifs_randesu_update_hist, hist));
+
+  return IGRAPH_SUCCESS;
+}
+
+/**
+ * \function igraph_motifs_randesu_callback
+ * \brief Finds motifs in a graph and calls a function for each of them
+ * 
+ * </para><para>
+ * Similarly to \ref igraph_motifs_randesu(), this function is able to find the
+ * different motifs of size three and four (ie. the number of different
+ * subgraphs with three and four vertices) in the network. However, instead of
+ * counting them, the function will call a callback function for each motif
+ * found to allow further tests or post-processing.
+ * 
+ * </para><para>
+ * The \c cut_prob argument also allows sampling the motifs, just like for
+ * \ref igraph_motifs_randesu(). Set the \c cut_prob argument to a zero vector
+ * for finding all motifs. 
+ * 
+ * \param graph The graph to find the motifs in.
+ * \param size The size of the motifs to search for. Only three and
+ *        four are implemented currently. The limitation is not in the
+ *        motif finding code, but the graph isomorphism code.
+ * \param cut_prob Vector of probabilities for cutting the search tree
+ *        at a given level. The first element is the first level, etc.
+ *        Supply all zeros here (of length \c size) to find all motifs 
+ *        in a graph.
+ * \param callback A pointer to a function of type \ref igraph_motifs_handler_t.
+ *        This function will be called whenever a new motif is found.
+ * \param extra Extra argument to pass to the callback function.
+ * \return Error code.
+ * 
+ * Time complexity: TODO.
+ * 
+ * \example examples/simple/igraph_motifs_randesu.c
+ */
+
+int igraph_motifs_randesu_callback(const igraph_t *graph, int size,
+		const igraph_vector_t *cut_prob, igraph_motifs_handler_t *callback,
+		void* extra) {
 
   long int no_of_nodes=igraph_vcount(graph);
   igraph_adjlist_t allneis, alloutneis;
@@ -97,10 +188,11 @@ int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
   long int *added;
   char *subg;
   
-  long int histlen;
   unsigned int *arr_idx, *arr_code;
   int code=0;
   unsigned char mul, idx;
+  
+  igraph_bool_t terminate = 0;
 
   if (size != 3 && size != 4) {
     IGRAPH_ERROR("Only 3 and 4 vertex motifs are implemented",
@@ -109,30 +201,23 @@ int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
   if (size==3) {
     mul=3;
     if (igraph_is_directed(graph)) {
-      histlen=16;
       arr_idx=igraph_i_isoclass_3_idx;
       arr_code=igraph_i_isoclass2_3;
     } else {
-      histlen=4;
       arr_idx=igraph_i_isoclass_3u_idx;
       arr_code=igraph_i_isoclass2_3u;
     }
   } else {
     mul=4;
     if (igraph_is_directed(graph)) {
-      histlen=218;
       arr_idx=igraph_i_isoclass_4_idx;
       arr_code=igraph_i_isoclass2_4;
     } else {
-      histlen=11;
       arr_idx=igraph_i_isoclass_4u_idx;
       arr_code=igraph_i_isoclass2_4u;
     }
   }
 
-  IGRAPH_CHECK(igraph_vector_resize(hist, histlen));
-  igraph_vector_null(hist);
-  
   added=igraph_Calloc(no_of_nodes, long int);
   if (added==0) {
     IGRAPH_ERROR("Cannot find motifs", IGRAPH_ENOMEM);
@@ -161,6 +246,11 @@ int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
     long int level;
 
     IGRAPH_ALLOW_INTERRUPTION();
+
+    if (VECTOR(*cut_prob)[0] == 1 ||
+	RNG_UNIF01() < VECTOR(*cut_prob)[0]) {
+      continue;
+    }
     
     /* init G */
     igraph_vector_clear(&vids); level=0;
@@ -213,35 +303,45 @@ int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
 	    }
 	  }
 
+	  if (callback(graph, &vids, arr_code[code], extra)) {
+	    terminate = 1;
+	    break;
+	  }
 	  igraph_vector_pop_back(&vids);
 	  subg[last]=0;
-	  VECTOR(*hist)[arr_code[code]] += 1;
 	}
       }
 
+      /* did the callback function asked us to terminate the search? */
+      if (terminate)
+	break;
+
       /* can we step down? */
       if (level < size-1 && 
-	  !igraph_vector_empty(&adjverts) && 
-	  (cp==0 || RNG_UNIF01() > cp)) {
-	/* yes, step down */
+	  !igraph_vector_empty(&adjverts)) {
+	/* we might step down */
 	long int neifather=igraph_vector_pop_back(&adjverts);
 	long int nei=igraph_vector_pop_back(&adjverts);
-	IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
-	subg[nei] = level+1; added[nei] += 1; level += 1;
 
-	IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
-	IGRAPH_CHECK(igraph_stack_push(&stack, nei));
-	IGRAPH_CHECK(igraph_stack_push(&stack, level));
-	
-	neis=igraph_adjlist_get(&allneis, nei);
-	s=igraph_vector_size(neis);
-	for (i=0; i<s; i++) {
-	  long int nei2=VECTOR(*neis)[i];
-	  if (!added[nei2] && nei2 > father) {
-	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
-	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
+	if (cp==0 || RNG_UNIF01() > cp) {
+	  /* yes, step down */
+	  IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
+	  subg[nei] = level+1; added[nei] += 1; level += 1;
+
+	  IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, nei));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, level));
+	  
+	  neis=igraph_adjlist_get(&allneis, nei);
+	  s=igraph_vector_size(neis);
+	  for (i=0; i<s; i++) {
+	    long int nei2=VECTOR(*neis)[i];
+	    if (!added[nei2] && nei2 > father) {
+	      IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
+	      IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
+	    }
+	    added[nei2] += 1;
 	  }
-	  added[nei2] += 1;
 	}
       } else {
 	/* no, step back */
@@ -271,6 +371,10 @@ int igraph_motifs_randesu(const igraph_t *graph, igraph_vector_t *hist,
       
     } /* while */
 
+    /* did the callback function asked us to terminate the search? */
+    if (terminate)
+      break;
+
     /* clear the added vector */
     added[father] -= 1;
     subg[father] = 0;
@@ -387,6 +491,11 @@ int igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_integer_t *est,
 
     IGRAPH_ALLOW_INTERRUPTION();
 
+    if (VECTOR(*cut_prob)[0] == 1 ||
+	RNG_UNIF01() < VECTOR(*cut_prob)[0]) {
+      continue;
+    }
+
     /* init G */
     igraph_vector_clear(&vids); level=0;
     IGRAPH_CHECK(igraph_vector_push_back(&vids, father));
@@ -398,7 +507,7 @@ int igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_integer_t *est,
     s=igraph_vector_size(&neis);
     for (i=0; i<s; i++) {
       long int nei=VECTOR(neis)[i];
-      if (!added[nei]) {
+      if (!added[nei] && nei > father) {
 	IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
 	IGRAPH_CHECK(igraph_vector_push_back(&adjverts, father));
       }
@@ -420,27 +529,30 @@ int igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_integer_t *est,
       }
 
       if (level < size-1 && 
-	  !igraph_vector_empty(&adjverts) && 
-	  (cp==0 || RNG_UNIF01() > cp)) {
-	/* yes, step down */
+	  !igraph_vector_empty(&adjverts)) {
+	/* We might step down */
 	long int neifather=igraph_vector_pop_back(&adjverts);
 	long int nei=igraph_vector_pop_back(&adjverts);
-	IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
-	added[nei] += 1; level += 1;
+	
+	if (cp==0 || RNG_UNIF01() > cp) {
+	  /* Yes, step down */
+	  IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
+	  added[nei] += 1; level += 1;
 
-	IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
-	IGRAPH_CHECK(igraph_stack_push(&stack, nei));
-	IGRAPH_CHECK(igraph_stack_push(&stack, level));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, nei));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, level));
 
-	IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
-	s=igraph_vector_size(&neis);
-	for (i=0; i<s; i++) {
-	  long int nei2=VECTOR(neis)[i];
-	  if (!added[nei2]) {
-	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
+	  IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+	  s=igraph_vector_size(&neis);
+	  for (i=0; i<s; i++) {
+	    long int nei2=VECTOR(neis)[i];
+	    if (!added[nei2] && nei2 > father) {
+	      IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
 	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
+	    }
+	    added[nei2] += 1;
 	  }
-	  added[nei2] += 1;
 	}
       } else {
 	/* no, step back */
@@ -483,7 +595,6 @@ int igraph_motifs_randesu_estimate(const igraph_t *graph, igraph_integer_t *est,
   RNG_END();
 
   (*est) *= ((double)no_of_nodes/sample_size);
-  (*est) /= size;
 
   if (parsample==0) {
     igraph_vector_destroy(sample);
@@ -557,7 +668,12 @@ int igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t *no,
   for (father=0; father<no_of_nodes; father++) {
     long int level, s;
 
-    IGRAPH_ALLOW_INTERRUPTION();
+    IGRAPH_ALLOW_INTERRUPTION();    
+
+    if (VECTOR(*cut_prob)[0] == 1 ||
+	RNG_UNIF01() < VECTOR(*cut_prob)[0]) {
+      continue;
+    }
 
     /* init G */
     igraph_vector_clear(&vids); level=0;
@@ -570,7 +686,7 @@ int igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t *no,
     s=igraph_vector_size(&neis);
     for (i=0; i<s; i++) {
       long int nei=VECTOR(neis)[i];
-      if (!added[nei]) {
+      if (!added[nei] && nei > father) {
 	IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
 	IGRAPH_CHECK(igraph_vector_push_back(&adjverts, father));
       }
@@ -592,27 +708,30 @@ int igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t *no,
       }
 
       if (level < size-1 && 
-	  !igraph_vector_empty(&adjverts) && 
-	  (cp==0 || RNG_UNIF01() > cp)) {
-	/* yes, step down */
+	  !igraph_vector_empty(&adjverts)) {
+	/* We might step down */
 	long int neifather=igraph_vector_pop_back(&adjverts);
 	long int nei=igraph_vector_pop_back(&adjverts);
-	IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
-	added[nei] += 1; level += 1;
 
-	IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
-	IGRAPH_CHECK(igraph_stack_push(&stack, nei));
-	IGRAPH_CHECK(igraph_stack_push(&stack, level));
-
-	IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
-	s=igraph_vector_size(&neis);
-	for (i=0; i<s; i++) {
-	  long int nei2=VECTOR(neis)[i];
-	  if (!added[nei2]) {
-	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
-	    IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
+	if (cp==0 || RNG_UNIF01() > cp) {
+	  /* Yes, step down */
+	  IGRAPH_CHECK(igraph_vector_push_back(&vids, nei));
+	  added[nei] += 1; level += 1;
+	  
+	  IGRAPH_CHECK(igraph_stack_push(&stack, neifather));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, nei));
+	  IGRAPH_CHECK(igraph_stack_push(&stack, level));
+	  
+	  IGRAPH_CHECK(igraph_neighbors(graph, &neis, nei, IGRAPH_ALL));
+	  s=igraph_vector_size(&neis);
+	  for (i=0; i<s; i++) {
+	    long int nei2=VECTOR(neis)[i];
+	    if (!added[nei2] && nei2 > father) {
+	      IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei2));
+	      IGRAPH_CHECK(igraph_vector_push_back(&adjverts, nei));
+	    }
+	    added[nei2] += 1;
 	  }
-	  added[nei2] += 1;
 	}
       } else {
 	/* no, step back */
@@ -654,8 +773,6 @@ int igraph_motifs_randesu_no(const igraph_t *graph, igraph_integer_t *no,
 
   RNG_END();
 
-  *no /= size;
-
   igraph_Free(added);
   igraph_vector_destroy(&vids);
   igraph_vector_destroy(&adjverts);
@@ -707,7 +824,8 @@ int igraph_dyad_census(const igraph_t *graph, igraph_integer_t *mut,
     IGRAPH_WARNING("Dyad census called on undirected graph");
   }
 
-  IGRAPH_CHECK(igraph_reciprocity(graph, &rec, 1));
+  IGRAPH_CHECK(igraph_reciprocity(graph, &rec, /*loops=*/1, 
+				  IGRAPH_RECIPROCITY_RATIO));
   *mut = rec*ec/(rec+1);
   *asym = ec-2*(*mut);
   *null = vc*(vc-1)/2-(*mut)-(*asym);
